<?php
namespace MediaFiles;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Settings {
	public static $enabled_post_types = [];
	public static $multiple_folders = false;
	public static $remove_from_all_folders = false;
	public static $media_library_infinite_scrolling = false;
	public static $filesizes = false;
	public static $folder_name_in_title = false;

	public function __construct() {
		add_filter( 'plugin_action_links_' . BWDMFMX_MEDIAFILES_BASENAME, [$this, 'plugin_settings_link'] );

		add_action( 'admin_init', [$this, 'register_settings'] );
		add_action( 'admin_menu', [$this, 'settings_menu'] );
		add_action( 'admin_enqueue_scripts', [$this, 'enqueue_settings_assets'] );

		add_action( 'wp_ajax_bwdmfmx_mediafiles_delete_plugin_data', [$this, 'delete_plugin_data'] );
	}
	
	public function plugin_settings_link( $actions ) {
		$actions[] = '<a href="' . admin_url( 'options-general.php?page=bwdmfmx_mediafiles_settings' ) . '">' . esc_html__( 'Settings', 'Bwdmediafiles' ) . '</a>';
		
		return $actions;
	}

	public function enqueue_settings_assets() {
		if ( get_current_screen() && get_current_screen()->base === 'settings_page_bwdmfmx_mediafiles_settings' ) {
			wp_enqueue_style( 'bwdmfmx-mediafiles-settings', BWDMFMX_MEDIAFILES_ASSETS_URL . '/css/settings.min.css', [], filemtime( BWDMFMX_MEDIAFILES_ASSETS_PATH .'/css/settings.min.css' ) );
			wp_enqueue_script( 'bwdmfmx-mediafiles-settings', BWDMFMX_MEDIAFILES_ASSETS_URL . '/js/settings.min.js', [], filemtime( BWDMFMX_MEDIAFILES_ASSETS_PATH .'/js/settings.min.js' ) );
			
			// MediaFiles - Settings 
			wp_localize_script( 'bwdmfmx-mediafiles-settings', 'MediaFiles', [
				'debug'   => isset( $_GET['debug'] ),
				'ajaxUrl' => admin_url( 'admin-ajax.php' ),
				'i18n'    => [
					'deletePluginData' => esc_html__( 'Are you sure you want to delete all your MediaFiles folders, associations, and settings?', 'Bwdmediafiles' ),
				]
			] );
		}
	}

	public function register_settings() {
		// General
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_default_open_folders' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_multiple_folders' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_remove_from_all_folders' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_media_library_infinite_scrolling' );
		

		// Permissions (per user role)
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_folder_access' );
  }

	/**
	 * Run in 'init' to use 'apply_filters'
	 */
	public static function get_settings() {
		// Get all post types enabled for MediaFiles
		self::get_enabled_post_types();

		self::$multiple_folders = get_option( 'bwdmfmx_mediafiles_multiple_folders', false );
		self::$remove_from_all_folders = get_option( 'bwdmfmx_mediafiles_remove_from_all_folders', false );
		self::$filesizes = get_option( 'bwdmfmx_mediafiles_filesizes', false );
		self::$folder_name_in_title = get_option( 'bwdmfmx_mediafiles_folder_name_in_title', false );
		
		self::$media_library_infinite_scrolling = get_option( 'bwdmfmx_mediafiles_media_library_infinite_scrolling', false );
		
		if ( self::$media_library_infinite_scrolling ) {
			add_filter( 'media_library_infinite_scrolling', '__return_true' );
		}
	}

	/**
	 * Get enabled post types
	 * 
	 * @since 1.8.1
	 */
	public static function get_enabled_post_types() {
		self::$enabled_post_types = ['attachment'];

		$enabled_post_types = get_option( 'bwdmfmx_mediafiles_post_types', [] );

		if ( is_array( $enabled_post_types ) ) {
			self::$enabled_post_types = array_merge( self::$enabled_post_types, $enabled_post_types );
		}

		self::$enabled_post_types = apply_filters( 'Bwdmediafiles/post_types', self::$enabled_post_types );

		return self::$enabled_post_types;
	}

	/**
	 * Delete all MediaFiles plugin data (folders, options entries, etc.)
	 */
	public function delete_plugin_data() {
		Ajax::verify_nonce();

		if ( ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error();
		}

		// @since 1.8: Delete all ZIP files in 'bwdmfmx-mediafiles-temp' directory (contains all generated ZIP files)
		$wp_upload_dir = wp_upload_dir();
		$temp_path = trailingslashit( $wp_upload_dir['basedir'] ) . BWDMFMX_MEDIAFILES_TEMP_DIR;

		if ( file_exists( $temp_path ) ) {
			$zip_files_deleted = array_map( 'unlink', glob( "$temp_path/*.zip*" ) );
		}

		$folders = get_terms( BWDMFMX_MEDIAFILES_TAXONOMY, ['hide_empty' => false] );

		// PRO: Get MediaFiles folders of all post types (term prefix: 'hf_cat_')
		global $wpdb;

		$bwdmfmx_mediafiles_folders = $wpdb->get_results(
			"SELECT * FROM " . $wpdb->term_taxonomy . "
			LEFT JOIN  " . $wpdb->terms . "
			ON  " . $wpdb->term_taxonomy . ".term_id =  " . $wpdb->terms . ".term_id
			WHERE " . $wpdb->term_taxonomy . ".taxonomy LIKE 'hf_cat_%'
			ORDER BY parent ASC"
		);

		$folders = array_merge( $folders, $bwdmfmx_mediafiles_folders );

		$folders_deleted = [];

		foreach ( $folders as $folder ) {
			$folder_id = intval( $folder->term_id );

			// Delete position (termmeta)
			delete_term_meta( 'term', $folder_id, BWDMFMX_MEDIAFILES_POSITION );

			// Delete taxonomy term (attachment)
			$folder_deleted = wp_delete_term( $folder_id, $folder->taxonomy );

			$folders_deleted[$folder_id] = $folder_deleted;
		}

		// Delete db options entries (starting with 'bwdmfmx_mediafiles_')
		$all_options = wp_load_alloptions();
		$bwdmfmx_mediafiles_options = [];
		$options_deleted = [];

		foreach ( array_keys( $all_options ) as $option_key ) {
			if ( substr( $option_key, 0, strlen( 'bwdmfmx_mediafiles_' ) ) === 'bwdmfmx_mediafiles_' ) {
				$bwdmfmx_mediafiles_options[] = $option_key;
				$option_deleted = delete_option( $option_key );

				if ( $option_deleted ) {
					$options_deleted[] = $option_key;
				}
			}
		}

		$message = '';

		if ( count( $folders_deleted ) || count( $options_deleted ) ) {
			$message = esc_html__( 'MediaFiles data has been deleted.', 'Bwdmediafiles' );
		}

		else if ( ! is_array( $folders ) || ! count( $folders ) ) {
			$message = esc_html__( 'No MediaFiles data to delete.', 'Bwdmediafiles' );
		}

		// STEP: Delete PRO data
		do_action( 'Bwdmediafiles/delete_plugin_data' );

		wp_send_json_success( [
			'folders'         => $folders,
			'foldersDeleted'  => $folders_deleted,
			'options'         => $bwdmfmx_mediafiles_options,
			'optionsDeleted'  => $options_deleted,
			'message'         => $message,
		] );
	}

	/**
	 * MediaFiles settings
	 */
	public function settings_menu() {
    add_options_page(
      'MediaFiles ' . esc_html__( 'Settings', 'Bwdmediafiles' ),
      'MediaFiles',
      'manage_options',
      BWDMFMX_MEDIAFILES_SETTINGS_GROUP,
      function() {
				require_once BWDMFMX_MEDIAFILES_PATH . 'includes/admin/settings.php';
			}
    );
  }
}