<?php
namespace MediaFiles;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Setup {
  public function __construct() {
		add_action( 'init', [$this, 'register_taxonomy'] );
		add_action( 'plugins_loaded', [$this, 'load_plugin_textdomain'] );
		
		add_action( 'admin_enqueue_scripts', [$this, 'has_media_modal'] );
		add_action( 'admin_enqueue_scripts', [$this, 'enqueue_scripts'] );
		add_action( 'wp_enqueue_scripts', [$this, 'enqueue_scripts'] );

		// Admin: Upload new media screen
		add_action( 'admin_enqueue_scripts', [$this, 'admin_new_media_screen'] );
		add_filter( 'pre-upload-ui', [$this, 'upload_ui_media_new'] );

		// Elementor: Enqueue MediaFiles assets
		add_action( 'elementor/editor/after_enqueue_scripts', [$this, 'enqueue_scripts'] );

		// Beaver builder (v2.7+ uses an iframe)
		add_action( 'fl_builder_ui_enqueue_scripts', [ $this, 'just_enqueue' ] );

		// BeTheme (v27+)
		add_action( 'mfn_header_enqueue', [ $this, 'just_enqueue' ] );

		// Cornerstone theme (v6+)
		add_action( 'cornerstone_before_wp_editor', [ $this, 'just_enqueue' ] );

		// Bricks: Disable lazy load in Gutenberg editor for MediaFiles gallery
		add_filter( 'wp_get_attachment_image_attributes', [ $this, 'set_image_attributes' ], 11, 3 );
	}

	/**
	 * Load MediaFiles without performing Data::$load_plugin check
	 * 
	 * Run on other plugin/theme actions like Beaver Builder, Cornerstone, etc.
	 * 
	 * @since 1.8.3
	 */
	public function just_enqueue() {
		Data::$load_plugin = true;
		
		$this->enqueue_scripts();
	}

	/**
	 * Register MediaFiles folders (custom taxonomy term; for post type: attachment)
	 */
	public function register_taxonomy() {
    register_taxonomy(
      BWDMFMX_MEDIAFILES_TAXONOMY,
      ['attachment'],
      [
				'labels' => [
					'name'               => esc_html__( 'Folder', 'Bwdmediafiles' ),
					'singular_name'      => esc_html__( 'Folder', 'Bwdmediafiles' ),
					'add_new_item'       => esc_html__( 'Add new folder', 'Bwdmediafiles' ),
					'edit_item'          => esc_html__( 'Edit folder', 'Bwdmediafiles' ),
					'new_item'           => esc_html__( 'Add new folder', 'Bwdmediafiles' ),
					'search_items'       => esc_html__( 'Search folder', 'Bwdmediafiles' ),
					'not_found'          => esc_html__( 'Folder not found', 'Bwdmediafiles' ),
					'not_found_in_trash' => esc_html__( 'Folder not found in trash', 'Bwdmediafiles' ),
				],
        'public'             => false,
        'publicly_queryable' => false,
        'hierarchical'       => true,
        'show_ui'            => true,
        'show_in_menu'       => false,
        'show_in_nav_menus'  => false,
				'show_in_quick_edit' => false,
				'show_in_rest'       => true, // wp.data.select('core').getEntityRecords('taxonomy', 'bwdmfmx_mediafiles_category', {per_page: -1}) // max 100 items for REST API call
        'show_admin_column'  => false,
        'rewrite'            => false,
        'update_count_callback' => '_update_generic_term_count', // Update term count for attachments
      ]
    );
  }

	/**
	 * Load plugin translation
	 * 
	 * Example: /languages/bwdmfmx-mediafiles-de_DE.mo
	 */
	public function load_plugin_textdomain() {
		load_plugin_textdomain( 'Bwdmediafiles', false, dirname( dirname( plugin_basename( __FILE__ ) ) ) . '/languages/' );
	}

	/**
	 * Load MediaFiles on any admin page that has 'media-modals' script enqueued
	 * 
	 * Skip post type & plugins admin screen in this check.
	 */
	public function has_media_modal() {
		// Return: No access
		if ( User::$folder_access === 'none' ) {
			return;
		}

		// Pro setting: MediaFiles disabled for files
		$disable_for_files = get_option( 'bwdmfmx_mediafiles_disable_for_files', false );

		if ( $disable_for_files && ( ! Data::$post_type || Data::$post_type === 'attachment' ) ) {
			Data::$load_plugin = false;

			return;
		}

		global $pagenow;

		// Media modal script is 'registered' (default 'enqueued' is not enough on custom ACF pages, etc.)
		if ( wp_script_is( 'media-editor', 'registered' ) && $pagenow !== 'edit.php' && $pagenow !== 'plugins.php' ) {
			Data::$load_plugin = true;
		}

		// WordPress Customizer (e.g. Set favicon)
		if ( $pagenow === 'customize.php' ) {
			Data::$load_plugin = true;
		}
	}

	/**
	 * Populate data when MediaFiles sidebar is mounted
	 * 
	 * To avoid running Data:.get_folders() when media modal has not been opened.
	 */
	public function enqueue_scripts() {
		if ( ! Data::$load_plugin ) {
			return;
		}

		global $pagenow;

		// Ensure window.wp.media exists for page builders
		wp_enqueue_media();

		wp_enqueue_style( 'Bwdmediafiles', BWDMFMX_MEDIAFILES_ASSETS_URL . '/css/media-files.min.css', [], filemtime( BWDMFMX_MEDIAFILES_ASSETS_PATH .'/css/media-files.min.css' ) );
		wp_enqueue_script( 'Bwdmediafiles', BWDMFMX_MEDIAFILES_ASSETS_URL . '/js/mediafiles.min.js', ['jquery'], filemtime( BWDMFMX_MEDIAFILES_ASSETS_PATH .'/js/mediafiles.min.js' ), true );
		
		$bwdmfmx_mediafiles_data = [
			'nonce'             => wp_create_nonce( 'bwdmfmx-mediafiles-nonce' ),
			'pagenow'           => $pagenow,
			'debug'             => isset( $_GET['debug'] ),
			'postType'          => Data::$post_type,
			'taxonomy'          => Data::$taxonomy,
			'folders'           => Data::$folders,
			'colors'            => Data::$colors,
			'openId'            => Data::$open_id,
			'filesizes'         => Settings::$filesizes,
			'folderNameInTitle' => Settings::$folder_name_in_title,
			'access'            => User::$folder_access,
			'svgSupport'        => User::$svg_support,
			'ajaxUrl'           => admin_url( 'admin-ajax.php' ),
			'assetsUrl'         => BWDMFMX_MEDIAFILES_ASSETS_URL,
			'i18n'              => [
				'infoCreateFolder'   => esc_html__( 'Click the "+" button above to create your first folder. Then start dragging items into it. To remove an item from a folder drag it into the "Uncategorized" folder.', 'Bwdmediafiles' ),
				'infoNoSearchResults'=> esc_html__( 'No folders found that match your search criteria. Please try another phrase or clear your search. Search is case-sensitive.', 'Bwdmediafiles' ),
				'attachmentIds'      => esc_html__( 'Attachment IDs', 'Bwdmediafiles' ),
				'items'              => esc_html__( 'Items', 'Bwdmediafiles' ),
				'folder'             => esc_html__( 'Folder', 'Bwdmediafiles' ),
				'folders'            => esc_html__( 'Folders', 'Bwdmediafiles' ),
				'select'             => esc_html__( 'Select', 'Bwdmediafiles' ),
				'create'             => esc_html__( 'Create', 'Bwdmediafiles' ),
				'cancel'             => esc_html__( 'Cancel', 'Bwdmediafiles' ),
				'rename'             => esc_html__( 'Rename', 'Bwdmediafiles' ),
				'delete'             => esc_html__( 'Delete', 'Bwdmediafiles' ),
				'move'               => esc_html__( 'Move', 'Bwdmediafiles' ),
				'search'             => esc_html__( 'Search', 'Bwdmediafiles' ),
				'uploaded'           => esc_html__( 'Uploaded', 'Bwdmediafiles' ),
				'sort'               => esc_html__( 'Sort', 'Bwdmediafiles' ),
				'expand'             => esc_html__( 'Expand', 'Bwdmediafiles' ),
				'collapse'           => esc_html__( 'Collapse', 'Bwdmediafiles' ),
				'details'            => esc_html__( 'Details', 'Bwdmediafiles' ),
				'duplicate'          => esc_html__( 'Duplicate', 'Bwdmediafiles' ),
				'clear'              => esc_html__( 'Clear', 'Bwdmediafiles' ),
				'subFolders'         => esc_html__( 'Subfolders', 'Bwdmediafiles' ),
				'foldersHelp'        => esc_html__( 'Hold down CTRL/CMD to select multiple folders.', 'Bwdmediafiles' ),

				'deleteFolders'      => esc_html__( 'Delete folders', 'Bwdmediafiles' ),
				'deleteSubfolders'   => esc_html__( 'Delete folders & subfolders', 'Bwdmediafiles' ),
				'deleteFoldersInfo'  => esc_html__( 'Please confirm how you\'d like to delete the MediaFiles folders listed below. All items inside those folders are safe & won\'t be deleted!', 'Bwdmediafiles' ),
				'foldersDeletedInfo' => esc_html__( 'The selected folders have been deleted.', 'Bwdmediafiles' ),
				'foldersDeletedInfo2'=> esc_html__( 'The selected folder no longer exists. We recommend selecting another folder.', 'Bwdmediafiles' ),

				'download'           => esc_html__( 'Download', 'Bwdmediafiles' ),

				// Folder colors
				'edit'               => esc_html__( 'Edit', 'Bwdmediafiles' ),
				'save'               => esc_html__( 'Save', 'Bwdmediafiles' ),
				'color'              => esc_html__( 'Color', 'Bwdmediafiles' ),

				// Shortcode generator
				'clickToCopy'        => esc_html__( 'Click to copy', 'Bwdmediafiles' ),
				'galleryShortcode'   => esc_html__( 'Gallery shortcode', 'Bwdmediafiles' ),
				'default'            => esc_html__( 'Default', 'Bwdmediafiles' ),
				'columns'            => esc_html__( 'Columns', 'Bwdmediafiles' ),
				'maxItems'           => esc_html__( 'Max items', 'Bwdmediafiles' ),
				'height'             => esc_html__( 'Height', 'Bwdmediafiles' ),
				'spacing'            => esc_html__( 'Spacing', 'Bwdmediafiles' ),
				'orderBy'            => esc_html__( 'Order by', 'Bwdmediafiles' ),
				'order'              => esc_html__( 'Order', 'Bwdmediafiles' ),
				'imageSize'          => esc_html__( 'Image size', 'Bwdmediafiles' ),
				'linkTo'             => esc_html__( 'Link to', 'Bwdmediafiles' ),
				'noCrop'             => esc_html__( 'No crop', 'Bwdmediafiles' ),
				'includeSubfolder'   => esc_html__( 'Include subfolder', 'Bwdmediafiles' ),
				'caption'            => esc_html__( 'Caption', 'Bwdmediafiles' ),
				'lightbox'           => esc_html__( 'Lightbox', 'Bwdmediafiles' ),
				'lightboxCaption'    => esc_html__( 'Lightbox', 'Bwdmediafiles' ) . ': ' . esc_html__( 'Caption', 'Bwdmediafiles' ),
				'lightboxFullscreen' => esc_html__( 'Lightbox', 'Bwdmediafiles' ) . ': ' . esc_html__( 'Fullscreen', 'Bwdmediafiles' ),
				'lightboxThumbnails' => esc_html__( 'Lightbox', 'Bwdmediafiles' ) . ': ' . esc_html__( 'Thumbnails', 'Bwdmediafiles' ),
				'lightboxZoom'       => esc_html__( 'Lightbox', 'Bwdmediafiles' ) . ': ' . esc_html__( 'Zoom', 'Bwdmediafiles' ),

				// Order by
				'date'     => esc_html__( 'Date', 'Bwdmediafiles' ),
				'author'   => esc_html__( 'Author', 'Bwdmediafiles' ),
				'name'     => esc_html__( 'Name', 'Bwdmediafiles' ),
				'title'    => esc_html__( 'Title', 'Bwdmediafiles' ),
				'type'     => esc_html__( 'Type', 'Bwdmediafiles' ),
				'modified' => esc_html__( 'Modified', 'Bwdmediafiles' ),
				'random'   => esc_html__( 'Random', 'Bwdmediafiles' ),
				'none'     => esc_html__( 'None', 'Bwdmediafiles' ),
				'filesize' => esc_html__( 'File size', 'Bwdmediafiles' ),

				// Order
				'descending' => esc_html__( 'Descending', 'Bwdmediafiles' ),
				'ascending' => esc_html__( 'Ascending', 'Bwdmediafiles' ),

				// Link to
				'attachment' => esc_html__( 'Attachment', 'Bwdmediafiles' ),
				'media'      => esc_html__( 'Media', 'Bwdmediafiles' ),
			],
		];

		// Filter MediaFiles data (filter plugins, etc.)
		$bwdmfmx_mediafiles_data = apply_filters( 'Bwdmediafiles/data', $bwdmfmx_mediafiles_data );
	
		wp_localize_script( 'Bwdmediafiles', 'MediaFiles', $bwdmfmx_mediafiles_data );
	}

	public function admin_new_media_screen() {
		global $pagenow;
		
		if ( $pagenow === 'media-new.php' ) {
			wp_enqueue_script( 'bwdmfmx-mediafiles-media-new', BWDMFMX_MEDIAFILES_ASSETS_URL . '/js/media-new.min.js', ['jquery'], filemtime( BWDMFMX_MEDIAFILES_ASSETS_PATH .'/js/media-new.min.js' ), true );
		}
	}

	/**
   * Add folder dropdown to media-new.php in WP admin
   */
  public function upload_ui_media_new() {
    if ( is_admin() && get_current_screen() && get_current_screen()->base === 'media' && in_array( User::$folder_access, ['upload', 'full'] ) ) {
      $tax_slug = Data::$taxonomy;
      $tax_obj = get_taxonomy( $tax_slug );

      echo '<div id="bwdmfmx-mediafiles-folder-upload-wrapper">';
      echo '<p>' . esc_html__( 'Optional: Assign a folder to your uploaded file(s):', 'Bwdmediafiles' ) . '</p>';

      echo '<p>';

      wp_dropdown_categories( [
        'id'               => 'bwdmfmx_mediafiles_category_add_new', // Has to be different from AJAX generated 'bwdmfmx_mediafiles_category' filter
        'show_option_none' => esc_html__( 'Uncategorized', 'Bwdmediafiles' ),
        'taxonomy'         => $tax_slug,
        'name'             => $tax_obj->name,
        'hierarchical'     => true,
				'hide_empty'       => false,
      ] );

      echo '</p>';
      echo '</div>';
    }
  }

	/**
	 * Bricks: Disable lazy load in Gutenberg editor for MediaFiles gallery
	 */
	function set_image_attributes( $attr, $attachment, $size ) {
		if ( ! empty( $_GET['attributes']['categories'] ) && ! empty( $attr['data-src'] ) ) {
			$attr['src'] = $attr['data-src'];
		}

		return $attr;
	}
}